package com.hyphenate.chatdemo;

import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;

import androidx.appcompat.app.AppCompatActivity;

import com.hyphenate.EMCallBack;
import com.hyphenate.EMChatRoomChangeListener;
import com.hyphenate.EMMultiDeviceListener;
import com.hyphenate.EMValueCallBack;
import com.hyphenate.chat.EMClient;
import com.hyphenate.chat.EMConversation;
import com.hyphenate.chat.EMPushManager;
import com.hyphenate.chat.EMSilentModeParam;
import com.hyphenate.chat.EMSilentModeResult;
import com.hyphenate.chatdemo.databinding.ActivityEmBinding;
import com.hyphenate.exceptions.HyphenateException;
import com.hyphenate.util.EMLog;

import java.util.List;
import java.util.Map;


public class EMActivity extends AppCompatActivity implements View.OnClickListener {
    private ActivityEmBinding mBinding;
    private static final String TAG = "EMActivity";
    private String conversationId = "250481334091777";
    private String chatRoomID = "251192217239553";
    private String username = "pu";
    private EMChatRoomChangeListener chatRoomChangeListener;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mBinding = ActivityEmBinding.inflate(getLayoutInflater());
        setContentView(mBinding.getRoot());
        initListener();
        initData();
    }

    private void initListener() {
        mBinding.btnSyncNoDisturb.setOnClickListener(this);
        mBinding.btnConversatonRemindType.setOnClickListener(this);
        mBinding.btnAddBlackList.setOnClickListener(this);
        mBinding.btnRemoveBlackList.setOnClickListener(this);
        mBinding.btnQueryBlackList.setOnClickListener(this);
        mBinding.btnAddContact.setOnClickListener(this);
        mBinding.btnRemoveContact.setOnClickListener(this);
        mBinding.btnAcceptInvitation.setOnClickListener(this);
        mBinding.btnRefuseInvitation.setOnClickListener(this);
        mBinding.btnQueryContacts.setOnClickListener(this);
        mBinding.btnQueryContactsFromServer.setOnClickListener(this);
        mBinding.btnQueryMsgCountWithDuration.setOnClickListener(this);
        mBinding.btnJoinChatroom.setOnClickListener(this);
        chatRoomChangeListener = new EMChatRoomChangeListener() {
            @Override
            public void onChatRoomDestroyed(String roomId, String roomName) {
                EMLog.e(TAG, "onChatRoomDestroyed roomId:" + roomId + " roomName:" + roomName);
            }

            @Override
            public void onMemberJoined(String roomId, String participant, String ext) {
                EMLog.e(TAG, "onMemberJoined roomId:" + roomId + " participant:" + participant + " ext:" + ext);
            }

            @Override
            public void onMemberExited(String roomId, String roomName, String participant) {
                EMLog.e(TAG, "onMemberExited roomId:" + roomId + " roomName:" + roomName + " participant:" + participant);
            }

            @Override
            public void onRemovedFromChatRoom(int reason, String roomId, String roomName, String participant) {
                EMLog.e(TAG, "onRemovedFromChatRoom reason:" + reason + " roomId:" + roomId + " roomName:" + roomName + " participant:" + participant);
            }

            @Override
            public void onMuteListAdded(String chatRoomId, List<String> mutes, long expireTime) {
                EMLog.e(TAG, "onMuteListAdded chatRoomId:" + chatRoomId + " mutes:" + mutes + " expireTime:" + expireTime);
            }

            @Override
            public void onMuteListRemoved(String chatRoomId, List<String> mutes) {
                EMLog.e(TAG, "onMuteListRemoved chatRoomId:" + chatRoomId + " mutes:" + mutes);
            }

            @Override
            public void onWhiteListAdded(String chatRoomId, List<String> whitelist) {

            }

            @Override
            public void onWhiteListRemoved(String chatRoomId, List<String> whitelist) {

            }

            @Override
            public void onAllMemberMuteStateChanged(String chatRoomId, boolean isMuted) {

            }

            @Override
            public void onAdminAdded(String chatRoomId, String admin) {

            }

            @Override
            public void onAdminRemoved(String chatRoomId, String admin) {

            }

            @Override
            public void onOwnerChanged(String chatRoomId, String newOwner, String oldOwner) {

            }

            @Override
            public void onAnnouncementChanged(String chatRoomId, String announcement) {

            }
        };
        EMClient.getInstance().chatroomManager().addChatRoomChangeListener(chatRoomChangeListener);
        mBinding.btnInput.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                username = s.toString();
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });
    }

    private void initData() {

    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btn_sync_no_disturb:
                syncNoDisturb();
                break;
            case R.id.btn_conversaton_remind_type:
                conversationRemindType();
                break;
            case R.id.btn_add_black_list:
                addBlacklist();
                break;
            case R.id.btn_remove_black_list:
                removeBlacklist();
                break;
            case R.id.btn_query_black_list:
                queryBlacklistLocal();
                break;
            case R.id.btn_add_contact:
                addContact();
                break;
            case R.id.btn_remove_contact:
                removeContact();
                break;
            case R.id.btn_accept_invitation:
                acceptInvitation();
                break;
            case R.id.btn_refuse_invitation:
                refuseInvitation();
                break;
            case R.id.btn_query_contacts:
                queryContacts();
                break;
            case R.id.btn_query_contacts_from_server:
                queryServerContacts();
                break;
            case R.id.btn_query_msg_count_with_duration:
                queryMsgCountWithDuration();
                break;
            case R.id.btn_join_chatroom:
                joinChatRoom();
                break;
        }
    }

    private void joinChatRoom() {
        String ext= "your ext info";
        boolean leaveOtherRooms=true;
        EMClient.getInstance().chatroomManager().joinChatRoom(chatRoomID,leaveOtherRooms,ext, new EMValueCallBack<com.hyphenate.chat.EMChatRoom>() {
            @Override
            public void onSuccess(com.hyphenate.chat.EMChatRoom value) {
                EMLog.i(TAG, "joinChatRoom onSuccess value:" + value);
            }

            @Override
            public void onError(int error, String errorMsg) {
                EMLog.i(TAG, "joinChatRoom onError error:" + error + " errorMsg:" + errorMsg);
            }
        });
    }

    private void queryMsgCountWithDuration() {
        String conversationId = "pu";
        EMConversation conversation = EMClient.getInstance().chatManager().getConversation(conversationId);
        if(conversation!=null) {
            long startTimestamp = System.currentTimeMillis() - 24 * 60 * 60 * 1000;
            int count = conversation.getAllMsgCount(startTimestamp, System.currentTimeMillis());
            EMLog.i(TAG, "queryMsgCountWithDuration count:" + count);
        }
    }

    private void queryServerContacts() {
        EMClient.getInstance().contactManager().asyncGetAllContactsFromServer(new EMValueCallBack<List<String>>() {
            @Override
            public void onSuccess(List<String> value) {
                EMLog.i(TAG, "queryServerContacts onSuccess value:" + value);
            }

            @Override
            public void onError(int error, String errorMsg) {
                EMLog.i(TAG, "queryServerContacts onError error:" + error + " errorMsg:" + errorMsg);
            }
        });
    }

    private void queryContacts() {
//        EMClient.getInstance().contactManager().asyncFetchAllContactsFromLocal(new EMValueCallBack<List<EMContact>>() {
//            @Override
//            public void onSuccess(List<EMContact> value) {
//                EMLog.i(TAG, "queryContacts onSuccess value:" + value);
//            }
//
//            @Override
//            public void onError(int error, String errorMsg) {
//                EMLog.i(TAG, "queryContacts onError error:" + error + " errorMsg:" + errorMsg);
//            }
//        });
        try {
            List<String> contactsFromLocal = EMClient.getInstance().contactManager().getContactsFromLocal();
            EMLog.i(TAG, "queryContacts contactsFromLocal:" + contactsFromLocal);
        } catch (HyphenateException e) {
            throw new RuntimeException(e);
        }
    }

    private void refuseInvitation() {
        EMClient.getInstance().contactManager().asyncDeclineInvitation(username, new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.i(TAG, "refuseInvitation onSuccess");
            }

            @Override
            public void onError(int code, String error) {
                EMLog.i(TAG, "refuseInvitation onError code:" + code + " error:" + error);
            }
        });
    }

    private void acceptInvitation() {
        EMClient.getInstance().contactManager().asyncAcceptInvitation(username, new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.i(TAG, "acceptInvitation onSuccess");
            }

            @Override
            public void onError(int code, String error) {
                EMLog.i(TAG, "acceptInvitation onError code:" + code + " error:" + error);
            }
        });
    }

    private void removeContact() {
        EMClient.getInstance().contactManager().asyncDeleteContact(username, new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.i(TAG, "removeContact onSuccess");
            }

            @Override
            public void onError(int code, String error) {
                EMLog.i(TAG, "removeContact onError code:" + code + " error:" + error);
            }
        });
    }

    private void addContact() {
        EMClient.getInstance().contactManager().asyncAddContact(username, "hello", new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.i(TAG, "addContact onSuccess");
            }

            @Override
            public void onError(int code, String error) {
                EMLog.i(TAG, "addContact onError code:" + code + " error:" + error);
            }
        });
    }

    private void queryBlacklistLocal() {
        List<String> blackListUsernames = EMClient.getInstance().contactManager().getBlackListUsernames();
        EMLog.i(TAG, "queryBlacklistLocal blackListUsernames:" + blackListUsernames);
    }

    private void removeBlacklist() {
        EMClient.getInstance().contactManager().asyncRemoveUserFromBlackList(username, new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.i(TAG, "removeBlacklist onSuccess");
            }

            @Override
            public void onError(int code, String error) {
                EMLog.i(TAG, "removeBlacklist onError code:" + code + " error:" + error);
            }
        });
    }

    private void addBlacklist() {
        EMClient.getInstance().contactManager().asyncAddUserToBlackList(username, false, new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.i(TAG, "addBlacklist onSuccess");
                queryBlacklistLocal();
            }

            @Override
            public void onError(int code, String error) {
                EMLog.i(TAG, "addBlacklist onError code:" + code + " error:" + error);
            }
        });
    }


    private void conversationRemindType() {
        Map<String, EMConversation> allConversations = EMClient.getInstance().chatManager().getAllConversations();
        EMLog.e(TAG, "conversationRemindType allConversations:" + allConversations.size());

        String conversationId = "pu";
        EMSilentModeParam emSilentModeParam = new EMSilentModeParam(EMSilentModeParam.EMSilentModeParamType.REMIND_TYPE);
        //设置免打扰类型
        emSilentModeParam.setRemindType(EMPushManager.EMPushRemindType.NONE);
        EMClient.getInstance().pushManager().setSilentModeForConversation(conversationId, EMConversation.EMConversationType.Chat, emSilentModeParam, new EMValueCallBack<EMSilentModeResult>() {
            @Override
            public void onSuccess(EMSilentModeResult value) {
                EMLog.i(TAG, "conversationRemindType onSuccess value:" + value);
            }

            @Override
            public void onError(int error, String errorMsg) {
                EMLog.i(TAG, "conversationRemindType onError error:" + error + " errorMsg:" + errorMsg);
            }
        });

        EMConversation conversation = EMClient.getInstance().chatManager().getConversation(conversationId);
        if(conversation!=null) {
            EMPushManager.EMPushRemindType emPushRemindType = conversation.pushRemindType();
            EMLog.i(TAG, "conversationRemindType emPushRemindType:" + emPushRemindType);
        }


    }

    private void syncNoDisturb() {
        EMClient.getInstance().pushManager().syncSilentModeConversationsFromServer(new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.i(TAG, "syncNoDisturb onSuccess");
            }

            @Override
            public void onError(int code, String error) {
                EMLog.i(TAG, "syncNoDisturb onError code:" + code + " error:" + error);
            }
        });


    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        EMClient.getInstance().chatroomManager().removeChatRoomListener(chatRoomChangeListener);
    }
}